/**
 * Copyright (c) 2021 Dominic Msters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "engine.h"

#include "display/primitives/quad.h"
#include "display/texture.h"
#include "display/shader.h"

camera_t *camera;
shader_t *shader;

primitive_t *primitive;
texture_t *texture;

engine_t * engineInit(platform_t *platform, char *name, uint32_t inputCount) {
  // Create the engine instance.
  engine_t *engine = malloc(sizeof(engine_t));
  if(engine == NULL) return NULL;

  engine->name = name;
  engine->platform = platform;

  // Setup the renderer.
  engine->render = renderInit(name);
  if(engine->render == NULL) {
    free(engine);
    return NULL;
  }

  // Setup the input manager.
  engine->input = inputInit(inputCount, platform->inputSourceCount);
  if(engine->input == NULL) {
    free(engine->render);
    free(engine);
    return NULL;
  }

  shader = assetShaderLoad("shaders/test.vert", "shaders/test.frag");

  camera = cameraCreate();
  cameraLookAt(camera,
    3, 3, 3,
    0, 0, 0
  );
  cameraPerspective(camera, 45.0f, 1920.0f/1080.0f, 3.0f, 100.0f);
  shaderUseCamera(shader, camera);

  // Test
  primitive = quadCreate(
    -1, -1, 0, 0,
     1,  1, 1, 1
  );

  texture = textureCreate(1, 1, NULL);
  pixel_t *pixels = malloc(sizeof(pixel_t) * 1 * 1);
  for(int32_t i = 0; i < 1*1; i++) {
    pixels[i].r = 0xFF, pixels[i].g = 0x00, pixels[i].b = 0xFF;
    pixels[i].a = 0xFF;
  }
  textureBufferPixels(texture, 0, 0, 1, 1, pixels);
  shaderUseTexture(shader, texture);

  return engine;
}

uint32_t engineUpdate(engine_t *engine) {
  shaderUse(shader);
  renderFrame(engine->render);
  
  primitiveDraw(primitive, 0, 6);

  inputUpdate(engine->input);

  return 0;
}

bool engineDispose(engine_t *engine) {
  if(!renderDispose(engine->render)) return false;
  free(engine);

  return true;
}